import os.path
from stem.control import Controller
from .utils import find_port

class HiddenService(object):

    def __init__(
            self,
            ports=None,
            control_port=9051,
            key_raw=None,
            key_file=None,
            await_publication=True):
        if ports is None:
            ports = {80: find_port()}
        self.ports = ports
        self.control_port = control_port
        self._key = None
        self._key_file = key_file
        if key_raw is not None:
            # Read key from string
            self._key = key_raw.split(':', 1)
        elif key_file is not None and os.path.exists(key_file):
            # Read key from file if file exists
            fp = open(key_file, 'r')
            self._key = fp.read().split(':', 1)
            fp.close()
        self.__start_service(await_publication)

    def __start_service(self, await_publication):
        # Create controller
        controller = Controller.from_port(
            address='127.0.0.1',
            port=self.control_port
        )
        controller.authenticate(password='')
        options = {}
        options['await_publication'] = await_publication
        if self._key is not None:
            options['key_type'] = self._key[0]
            options['key_content'] = self._key[1]
        # Create hidden service
        service = controller.create_ephemeral_hidden_service(
            self.ports,
            **options
        )
        if self._key is None:
            self._key = (service.private_key_type, service.private_key)
        # Write key if a file is provided
        if self._key_file is not None and not os.path.exists(self._key_file):
            self.save_key(self._key_file)
        self.service = service
        self.onion = '{}.onion'.format(service.service_id)

    def get_key(self):
        return '{}:{}'.format(self._key[0], self._key[1])

    def save_key(self, filename):
        fp = open(filename, 'w')
        fp.write(self.get_key())
        fp.close()


def find_port(start=1024, end=49151):
    ''' Loop over ports looking for an open one '''
    from socket import socket
    sock = socket()
    for port in range(start, end):
        try:
            sock.bind(('127.0.0.1', port))
        except:
            continue
        return port
    raise Error('No ports available')



import shh
import time
from argparse import ArgumentParser

parser = ArgumentParser()
parser.add_argument(
    '-p',
    '--port',
    default=None,
    type=str,
    help='local port to serve through Tor (otherwise random)',
)
parser.add_argument(
    '-k',
    '--key',
    default=None,
    type=str,
    help='key file to use (will generate if not found)',
)

parser.add_argument(
    '-c',
    '--control_port',
    default=9051,
    type=int,
    help='control port for Tor instance',
)

args = parser.parse_args()

if args.port is None:
    port = shh.utils.find_port()
else:
    port = args.port
print('Local port: {}'.format(port))

control_port = args.control_port

print('Creating hidden service...')
hidden = shh.HiddenService(
    ports={80: port},
    key_file=args.key,
    control_port=control_port
)
print('Serving at: ' + hidden.onion)

while True:
    time.sleep(10)



'''
shh
Project page: https://github.com/wybiral/shh
'''

from . import utils
from .hidden_service import HiddenService
from .proxy import Proxy

__author__ = 'Davy Wybiral (davy.wybiral@gmail.com)'
__version__ = '0.9.0'



from setuptools import setup

version = '0.9.0'

setup(
    name='shh',
    version=version,
    url='https://github.com/wybiral/shh/',
    author='Davy Wybiral',
    author_email='davy.wybiral@gmail.com',
    description='Making Tor hidden services easy',
    keywords = 'tor onion hidden service',
    packages=['shh'],
    platforms='any',
    install_requires=[
        'pysocks==1.6.7',
        'stem==1.5.4',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Security',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



import socket
import socks

class Proxy:

    def __init__(self, host='127.0.0.1', port=9050):
        self.host = host
        self.port = port

    def __enter__(self):
        # Save original methods
        self._socket = socket.socket
        self._getaddrinfo = socket.getaddrinfo
        # Configure socks proxy
        socks.set_default_proxy(socks.SOCKS5, self.host, self.port)
        socket.socket = socks.socksocket
        # Patch domain lookup
        socket.getaddrinfo = _getaddrinfo

    def __exit__(self, type, value, traceback):
        # Restore original methods
        socket.socket = self._socket
        socket.getaddrinfo = self._getaddrinfo


def _getaddrinfo(host, port, *args):
    # https://docs.python.org/3/library/socket.html#socket.getaddrinfo
    sockaddr = (host, port)
    return [(socket.AF_INET, socket.SOCK_STREAM, 6, '', sockaddr)]
